@@ -6,7 +6,7 @@ use HTML::TreeBuilder;
 use base qw(HTML::FormatText);
 use vars qw($VERSION);
 
-$VERSION = '0.03';
+$VERSION = '0.04';
 
 sub new {
 
@@ -59,13 +59,15 @@ sub a_start {
     my $node = shift;
     # local urls are no use so we have to make them absolute
     my $href = $node->attr('href') || '';
-    if ($href =~ m#^http:|^mailto:#) {
-        push @{$self->{_links}}, $href;
-    } else {
-        my $u = URI::WithBase->new($href, $self->{base});
-        push @{$self->{_links}}, $u->abs();
+    if ( $href ) {
+        if ($href =~ m#^http:|^mailto:#) {
+            push @{$self->{_links}}, $href;
+        } else {
+            my $u = URI::WithBase->new($href, $self->{base});
+            push @{$self->{_links}}, $u->abs();
+        }
+        $self->out( $self->text('before_link') );
     }
-    $self->out( $self->text('before_link') );
     $self->SUPER::a_start();
 
 }
@@ -128,7 +130,7 @@ sub html_end {
 
 }
 
-sub link_num {
+sub _link_num {
 
     my ($self, $num) = @_;
     $num = $#{$self->{_links}} unless (defined $num);
@@ -141,7 +143,7 @@ sub text {
     my ($self, $type, $num, $href) = @_;
     $href = $self->{_links}->[$#{$self->{_links}}]
             unless (defined $num and defined $href);
-    $num = $self->link_num($num);
+    $num = $self->_link_num($num);
     my $text = $self->{$type};
     $text =~ s/%n/$num/g;
     $text =~ s/%l/$href/g;
@@ -153,19 +155,12 @@ sub parse {
     my $self = shift;
     my $text = shift;
 
+    return undef unless defined $text;
+    return '' if $text eq '';
+
     my $tree = HTML::TreeBuilder->new->parse( $text );
 
-    unless ( $tree ) {
-        $self->error("HTML::TreeBuilder problem" . $! ? ": $!" : '');
-        return undef;
-    }
-
-    $tree->eof();
-    my $return_text = $self->format( $tree );
-
-    $tree->delete;
-
-    return $return_text;
+    return $self->_parse( $tree );
 }
 
 sub parse_file {
@@ -180,8 +175,16 @@ sub parse_file {
 
     my $tree = HTML::TreeBuilder->new->parse_file( $file );
     
+    return $self->_parse( $tree );
+}
+
+sub _parse {
+
+    my $self = shift;
+    my $tree = shift;
+
     unless ( $tree ) {
-        $self->error("HTML::TreeBuilder problem" . $! ? ": $!" : '');
+        $self->error( "HTML::TreeBuilder problem" . ( $! ? ": $!" : '' ) );
         return undef;
     }
     $tree->eof();
@@ -192,6 +195,7 @@ sub parse_file {
 
     return $return_text;
 }
+    
 
 sub error {
     my $self = shift;
@@ -317,6 +321,9 @@ If set to 1 then italicised text will be surrounded by / and bolded text by _.
 
 Takes some HTML and returns it as text. Returns undef on error.
 
+Will also return undef if you don't pass it undef. Returns an empty 
+string if passed an empty string.
+
 =head2 parse_file
 
     my $text = $f->parse_file($filename);
@@ -338,6 +345,11 @@ When passing HTML fragments the results may be a little unpredictable.
 I've tried to work round the most egregious of the issues but any 
 unexpected results are welcome. 
 
+Also note that if for some reason there is an a tag in the document
+that does not have an href attribute then it will be quietly ignored.
+If this is really a problem for anyone then let me know and I'll see
+if I can think of a sensible thing to do in this case.
+
 =head1 AUTHOR
 
 Struan Donald. E<lt>struan@cpan.orgE<gt>
